@@ -19,10 +19,12 @@ typedef struct _rule_t {
         int msg_urgency;
 
         /* actions */
-        int timeout;
-        int urgency;
+        gint64 timeout;
+        enum urgency urgency;
         enum markup_mode markup;
         int history_ignore;
+        int match_transient;
+        int set_transient;
         char *new_icon;
         char *fg;
         char *bg;
@@ -32,10 +34,10 @@ typedef struct _rule_t {
 
 extern GSList *rules;
 
-void rule_init(rule_t * r);
-void rule_apply(rule_t * r, notification * n);
-void rule_apply_all(notification * n);
-bool rule_matches_notification(rule_t * r, notification * n);
+void rule_init(rule_t *r);
+void rule_apply(rule_t *r, notification *n);
+void rule_apply_all(notification *n);
+bool rule_matches_notification(rule_t *r, notification *n);
 
 #endif
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
