@@ -47,25 +47,24 @@ static enum markup_mode parse_markup_mode(const char *mode)
         }
 }
 
-static int ini_get_urgency(char *section, char *key, int def)
+static enum urgency ini_get_urgency(const char *section, const char *key, const int def)
 {
         int ret = def;
         char *urg = ini_get_string(section, key, "");
 
         if (strlen(urg) > 0) {
                 if (strcmp(urg, "low") == 0)
-                        ret = LOW;
+                        ret = URG_LOW;
                 else if (strcmp(urg, "normal") == 0)
-                        ret = NORM;
+                        ret = URG_NORM;
                 else if (strcmp(urg, "critical") == 0)
-                        ret = CRIT;
+                        ret = URG_CRIT;
                 else
                         fprintf(stderr,
                                 "unknown urgency: %s, ignoring\n",
                                 urg);
         }
-        if (urg)
-                g_free(urg);
+        g_free(urg);
         return ret;
 }
 
@@ -79,7 +78,18 @@ void load_settings(char *cmdline_config_path)
         xdgInitHandle(&xdg);
 
         if (cmdline_config_path != NULL) {
-                config_file = fopen(cmdline_config_path, "r");
+                if (0 == strcmp(cmdline_config_path, "-")) {
+                        config_file = stdin;
+                } else {
+                        config_file = fopen(cmdline_config_path, "r");
+                }
+
+                if(!config_file) {
+                        char *msg = g_strdup_printf(
+                                        "Cannot find config file: '%s'\n",
+                                        cmdline_config_path);
+                        die(msg, 1);
+                }
         }
         if (config_file == NULL) {
                 config_file = xdgConfigOpen("dunst/dunstrc", "r", &xdg);
@@ -95,6 +105,9 @@ void load_settings(char *cmdline_config_path)
         }
 
         load_ini_file(config_file);
+#else
+        fprintf(stderr, "Warning: dunstrc parsing disabled. "
+                        "Using STATIC_CONFIG is deprecated behavior.\n");
 #endif
 
         settings.per_monitor_dpi = option_get_bool(
@@ -111,22 +124,13 @@ void load_settings(char *cmdline_config_path)
 
         settings.font = option_get_string(
                 "global",
-                "font", "-font/-fn", font,
+                "font", "-font/-fn", defaults.font,
                 "The font dunst should use."
         );
 
         {
-                //If markup isn't set, fall back to allow_markup for backwards compatibility
-                if (ini_is_set("global", "markup") || cmdline_is_set("-markup")) {
-                        char *c = option_get_string(
-                                "global",
-                                "markup", "-markup", markup,
-                                "Specify how markup should be handled"
-                        );
-
-                        settings.markup = parse_markup_mode(c);
-                        g_free(c);
-                } else if (ini_is_set("global", "allow_markup")) {
+                // Check if allow_markup set
+                if (ini_is_set("global", "allow_markup")) {
                         bool allow_markup = option_get_bool(
                                 "global",
                                 "allow_markup", NULL, false,
@@ -135,50 +139,86 @@ void load_settings(char *cmdline_config_path)
 
                         settings.markup = (allow_markup ? MARKUP_FULL : MARKUP_STRIP);
                         fprintf(stderr, "Warning: 'allow_markup' is deprecated, please use 'markup' instead.\n");
-                } else {
-                        settings.markup = parse_markup_mode(markup); // None are set, parse the default value from config.h
                 }
+
+                char *c = option_get_string(
+                        "global",
+                        "markup", "-markup", NULL,
+                        "Specify how markup should be handled"
+                );
+
+                //Use markup if set
+                //Use default if settings.markup not set yet
+                //  (=>c empty&&!allow_markup)
+                if (c) {
+                        settings.markup = parse_markup_mode(c);
+                } else if (!settings.markup) {
+                        settings.markup = defaults.markup;
+                }
+                g_free(c);
         }
 
         settings.format = option_get_string(
                 "global",
-                "format", "-format", format,
+                "format", "-format", defaults.format,
                 "The format template for the notifications"
         );
 
         settings.sort = option_get_bool(
                 "global",
-                "sort", "-sort", sort,
+                "sort", "-sort", defaults.sort,
                 "Sort notifications by urgency and date?"
         );
 
         settings.indicate_hidden = option_get_bool(
                 "global",
-                "indicate_hidden", "-indicate_hidden", indicate_hidden,
+                "indicate_hidden", "-indicate_hidden", defaults.indicate_hidden,
                 "Show how many notificaitons are hidden?"
         );
 
         settings.word_wrap = option_get_bool(
                 "global",
-                "word_wrap", "-word_wrap", word_wrap,
+                "word_wrap", "-word_wrap", defaults.word_wrap,
                 "Truncating long lines or do word wrap"
         );
 
+        {
+                char *c = option_get_string(
+                        "global",
+                        "ellipsize", "-ellipsize", "",
+                        "Ellipsize truncated lines on the start/middle/end"
+                );
+
+                if (strlen(c) == 0) {
+                        settings.ellipsize = defaults.ellipsize;
+                } else if (strcmp(c, "start") == 0) {
+                        settings.ellipsize = start;
+                } else if (strcmp(c, "middle") == 0) {
+                        settings.ellipsize = middle;
+                } else if (strcmp(c, "end") == 0) {
+                        settings.ellipsize = end;
+                } else {
+                        fprintf(stderr, "Warning: unknown ellipsize value: \"%s\"\n", c);
+                        settings.ellipsize = defaults.ellipsize;
+                }
+                g_free(c);
+        }
+
         settings.ignore_newline = option_get_bool(
                 "global",
-                "ignore_newline", "-ignore_newline", ignore_newline,
+                "ignore_newline", "-ignore_newline", defaults.ignore_newline,
                 "Ignore newline characters in notifications"
         );
 
-        settings.idle_threshold = option_get_int(
+        settings.idle_threshold = option_get_time(
                 "global",
-                "idle_threshold", "-idle_threshold", idle_threshold,
+                "idle_threshold", "-idle_threshold", defaults.idle_threshold,
                 "Don't timeout notifications if user is longer idle than threshold"
         );
 
         settings.monitor = option_get_int(
                 "global",
-                "monitor", "-mon/-monitor", monitor,
+                "monitor", "-mon/-monitor", defaults.monitor,
                 "On which monitor should the notifications be displayed"
         );
 
@@ -197,37 +237,37 @@ void load_settings(char *cmdline_config_path)
 
         settings.title = option_get_string(
                 "global",
-                "title", "-t/-title", title,
+                "title", "-t/-title", defaults.title,
                 "Define the title of windows spawned by dunst."
         );
 
         settings.class = option_get_string(
                 "global",
-                "class", "-c/-class", class,
+                "class", "-c/-class", defaults.class,
                 "Define the class of windows spawned by dunst."
         );
 
         settings.geom = option_get_string(
                 "global",
-                "geometry", "-geom/-geometry", geom,
+                "geometry", "-geom/-geometry", defaults.geom,
                 "Geometry for the window"
         );
 
         settings.shrink = option_get_bool(
                 "global",
-                "shrink", "-shrink", shrink,
+                "shrink", "-shrink", defaults.shrink,
                 "Shrink window if it's smaller than the width"
         );
 
         settings.line_height = option_get_int(
                 "global",
-                "line_height", "-lh/-line_height", line_height,
+                "line_height", "-lh/-line_height", defaults.line_height,
                 "Add spacing between lines of text"
         );
 
         settings.notification_height = option_get_int(
                 "global",
-                "notification_height", "-nh/-notification_height", notification_height,
+                "notification_height", "-nh/-notification_height", defaults.notification_height,
                 "Define height of the window"
         );
 
@@ -252,9 +292,9 @@ void load_settings(char *cmdline_config_path)
                 }
         }
 
-        settings.show_age_threshold = option_get_int(
+        settings.show_age_threshold = option_get_time(
                 "global",
-                "show_age_threshold", "-show_age_threshold", show_age_threshold,
+                "show_age_threshold", "-show_age_threshold", defaults.show_age_threshold,
                 "When should the age of the notification be displayed?"
         );
 
@@ -266,43 +306,43 @@ void load_settings(char *cmdline_config_path)
 
         settings.sticky_history = option_get_bool(
                 "global",
-                "sticky_history", "-sticky_history", sticky_history,
+                "sticky_history", "-sticky_history", defaults.sticky_history,
                 "Don't timeout notifications popped up from history"
         );
 
         settings.history_length = option_get_int(
                 "global",
-                "history_length", "-history_length", history_length,
+                "history_length", "-history_length", defaults.history_length,
                 "Max amount of notifications kept in history"
         );
 
         settings.show_indicators = option_get_bool(
                 "global",
-                "show_indicators", "-show_indicators", show_indicators,
+                "show_indicators", "-show_indicators", defaults.show_indicators,
                 "Show indicators for actions \"(A)\" and URLs \"(U)\""
         );
 
         settings.separator_height = option_get_int(
                 "global",
-                "separator_height", "-sep_height/-separator_height", separator_height,
+                "separator_height", "-sep_height/-separator_height", defaults.separator_height,
                 "height of the separator line"
         );
 
         settings.padding = option_get_int(
                 "global",
-                "padding", "-padding", padding,
+                "padding", "-padding", defaults.padding,
                 "Padding between text and separator"
         );
 
         settings.h_padding = option_get_int(
                 "global",
-                "horizontal_padding", "-horizontal_padding", h_padding,
+                "horizontal_padding", "-horizontal_padding", defaults.h_padding,
                 "horizontal padding"
         );
 
         settings.transparency = option_get_int(
                 "global",
-                "transparency", "-transparency", transparency,
+                "transparency", "-transparency", defaults.transparency,
                 "Transparency. range 0-100"
         );
 
@@ -324,8 +364,8 @@ void load_settings(char *cmdline_config_path)
                                 settings.sep_color = CUSTOM;
                                 settings.sep_custom_color_str = g_strdup(c);
                         }
-                        g_free(c);
                 }
+                g_free(c);
         }
 
         settings.stack_duplicates = option_get_bool(
@@ -340,9 +380,9 @@ void load_settings(char *cmdline_config_path)
                 "print notification on startup"
         );
 
-        settings.dmenu = option_get_string(
+        settings.dmenu = option_get_path(
                 "global",
-                "dmenu", "-dmenu", dmenu,
+                "dmenu", "-dmenu", defaults.dmenu,
                 "path to dmenu"
         );
 
@@ -357,9 +397,9 @@ void load_settings(char *cmdline_config_path)
         }
 
 
-        settings.browser = option_get_string(
+        settings.browser = option_get_path(
                 "global",
-                "browser", "-browser", browser,
+                "browser", "-browser", defaults.browser,
                 "path to browser"
         );
 
@@ -386,63 +426,74 @@ void load_settings(char *cmdline_config_path)
 
         settings.max_icon_size = option_get_int(
                 "global",
-                "max_icon_size", "-max_icon_size", max_icon_size,
+                "max_icon_size", "-max_icon_size", defaults.max_icon_size,
                 "Scale larger icons down to this size, set to 0 to disable"
         );
 
-        settings.icon_folders = option_get_string(
+        // If the deprecated icon_folders option is used,
+        // read it and generate its usage string.
+        if (ini_is_set("global", "icon_folders") || cmdline_is_set("-icon_folders")) {
+                settings.icon_path = option_get_string(
+                        "global",
+                        "icon_folders", "-icon_folders", defaults.icon_path,
+                        "folders to default icons (deprecated, please use 'icon_path' instead)"
+                );
+                fprintf(stderr, "Warning: 'icon_folders' is deprecated, please use 'icon_path' instead.\n");
+        }
+        // Read value and generate usage string for icon_path.
+        // If icon_path is set, override icon_folder.
+        // if not, but icon_folder is set, use that instead of the compile time default.
+        settings.icon_path = option_get_string(
                 "global",
-                "icon_folders", "-icon_folders", icon_folders,
+                "icon_path", "-icon_path",
+                settings.icon_path ? settings.icon_path : defaults.icon_path,
                 "paths to default icons"
         );
 
         {
                 // Backwards compatibility with the legacy 'frame' section.
-
-                if (ini_is_set("global", "frame_width")) {
-                        settings.frame_width = option_get_int(
-                                "global",
-                                "frame_width", "-frame_width", frame_width,
-                                "Width of frame around the window"
-                        );
-                } else {
-                        if (ini_is_set("frame", "width")) {
-                                fprintf(stderr, "Warning: The frame section is deprecated, width has been renamed to frame_width and moved to the global section.\n");
-                        }
+                if (ini_is_set("frame", "width")) {
                         settings.frame_width = option_get_int(
                                 "frame",
-                                "width", "-frame_width", frame_width,
+                                "width", NULL, defaults.frame_width,
                                 "Width of frame around the window"
                         );
+                        fprintf(stderr, "Warning: The frame section is deprecated, width has been renamed to frame_width and moved to the global section.\n");
                 }
 
-                if (ini_is_set("global", "frame_color")) {
-                        settings.frame_color = option_get_string(
-                                "global",
-                                "frame_color", "-frame_color", frame_color,
-                                "Color of the frame around the window"
-                        );
-                } else {
-                        if (ini_is_set("frame", "color")) {
-                                fprintf(stderr, "Warning: The frame section is deprecated, color has been renamed to frame_color and moved to the global section.\n");
-                        }
+                settings.frame_width = option_get_int(
+                        "global",
+                        "frame_width", "-frame_width",
+                        settings.frame_width ? settings.frame_width : defaults.frame_width,
+                        "Width of frame around the window"
+                );
+
+                if (ini_is_set("frame", "color")) {
                         settings.frame_color = option_get_string(
                                 "frame",
-                                "color", "-frame_color", frame_color,
+                                "color", NULL, defaults.frame_color,
                                 "Color of the frame around the window"
                         );
+                        fprintf(stderr, "Warning: The frame section is deprecated, color has been renamed to frame_color and moved to the global section.\n");
                 }
 
+                settings.frame_color = option_get_string(
+                        "global",
+                        "frame_color", "-frame_color",
+                        settings.frame_color ? settings.frame_color : defaults.frame_color,
+                        "Color of the frame around the window"
+                );
+
         }
         settings.lowbgcolor = option_get_string(
                 "urgency_low",
-                "background", "-lb", lowbgcolor,
+                "background", "-lb", defaults.lowbgcolor,
                 "Background color for notifications with low urgency"
         );
 
         settings.lowfgcolor = option_get_string(
                 "urgency_low",
-                "foreground", "-lf", lowfgcolor,
+                "foreground", "-lf", defaults.lowfgcolor,
                 "Foreground color for notifications with low urgency"
         );
 
@@ -452,27 +503,27 @@ void load_settings(char *cmdline_config_path)
                 "Frame color for notifications with low urgency"
         );
 
-        settings.timeouts[LOW] = option_get_int(
+        settings.timeouts[URG_LOW] = option_get_time(
                 "urgency_low",
-                "timeout", "-lto", timeouts[LOW],
+                "timeout", "-lto", defaults.timeouts[URG_LOW],
                 "Timeout for notifications with low urgency"
         );
 
-        settings.icons[LOW] = option_get_string(
+        settings.icons[URG_LOW] = option_get_string(
                 "urgency_low",
-                "icon", "-li", icons[LOW],
+                "icon", "-li", defaults.icons[URG_LOW],
                 "Icon for notifications with low urgency"
         );
 
         settings.normbgcolor = option_get_string(
                 "urgency_normal",
-                "background", "-nb", normbgcolor,
+                "background", "-nb", defaults.normbgcolor,
                 "Background color for notifications with normal urgency"
         );
 
         settings.normfgcolor = option_get_string(
                 "urgency_normal",
-                "foreground", "-nf", normfgcolor,
+                "foreground", "-nf", defaults.normfgcolor,
                 "Foreground color for notifications with normal urgency"
         );
 
@@ -482,27 +533,27 @@ void load_settings(char *cmdline_config_path)
                 "Frame color for notifications with normal urgency"
         );
 
-        settings.timeouts[NORM] = option_get_int(
+        settings.timeouts[URG_NORM] = option_get_time(
                 "urgency_normal",
-                "timeout", "-nto", timeouts[NORM],
+                "timeout", "-nto", defaults.timeouts[URG_NORM],
                 "Timeout for notifications with normal urgency"
         );
 
-        settings.icons[NORM] = option_get_string(
+        settings.icons[URG_NORM] = option_get_string(
                 "urgency_normal",
-                "icon", "-ni", icons[NORM],
+                "icon", "-ni", defaults.icons[URG_NORM],
                 "Icon for notifications with normal urgency"
         );
 
         settings.critbgcolor = option_get_string(
                 "urgency_critical",
-                "background", "-cb", critbgcolor,
+                "background", "-cb", defaults.critbgcolor,
                 "Background color for notifications with critical urgency"
         );
 
         settings.critfgcolor = option_get_string(
                 "urgency_critical",
-                "foreground", "-cf", critfgcolor,
+                "foreground", "-cf", defaults.critfgcolor,
                 "Foreground color for notifications with ciritical urgency"
         );
 
@@ -512,39 +563,39 @@ void load_settings(char *cmdline_config_path)
                 "Frame color for notifications with critical urgency"
         );
 
-        settings.timeouts[CRIT] = option_get_int(
+        settings.timeouts[URG_CRIT] = option_get_time(
                 "urgency_critical",
-                "timeout", "-cto", timeouts[CRIT],
+                "timeout", "-cto", defaults.timeouts[URG_CRIT],
                 "Timeout for notifications with critical urgency"
         );
 
-        settings.icons[CRIT] = option_get_string(
+        settings.icons[URG_CRIT] = option_get_string(
                 "urgency_critical",
-                "icon", "-ci", icons[CRIT],
+                "icon", "-ci", defaults.icons[URG_CRIT],
                 "Icon for notifications with critical urgency"
         );
 
         settings.close_ks.str = option_get_string(
                 "shortcuts",
-                "close", "-key", close_ks.str,
+                "close", "-key", defaults.close_ks.str,
                 "Shortcut for closing one notification"
         );
 
         settings.close_all_ks.str = option_get_string(
                 "shortcuts",
-                "close_all", "-all_key", close_all_ks.str,
+                "close_all", "-all_key", defaults.close_all_ks.str,
                 "Shortcut for closing all notifications"
         );
 
         settings.history_ks.str = option_get_string(
                 "shortcuts",
-                "history", "-history_key", history_ks.str,
+                "history", "-history_key", defaults.history_ks.str,
                 "Shortcut to pop the last notification from history"
         );
 
         settings.context_ks.str = option_get_string(
                 "shortcuts",
-                "context", "-context_key", context_ks.str,
+                "context", "-context_key", defaults.context_ks.str,
                 "Shortcut for context menu"
         );
 
@@ -560,11 +611,11 @@ void load_settings(char *cmdline_config_path)
         );
 
         /* push hardcoded default rules into rules list */
-        for (int i = 0; i < LENGTH(default_rules); i++) {
+        for (int i = 0; i < G_N_ELEMENTS(default_rules); i++) {
                 rules = g_slist_insert(rules, &(default_rules[i]), -1);
         }
 
-        char *cur_section = NULL;
+        const char *cur_section = NULL;
         for (;;) {
                 cur_section = next_section(cur_section);
                 if (!cur_section)
@@ -580,7 +631,7 @@ void load_settings(char *cmdline_config_path)
 
                 /* check for existing rule with same name */
                 rule_t *r = NULL;
-                for (GSList * iter = rules; iter; iter = iter->next) {
+                for (GSList *iter = rules; iter; iter = iter->next) {
                         rule_t *match = iter->data;
                         if (match->name &&
                             strcmp(match->name, cur_section) == 0)
@@ -599,7 +650,7 @@ void load_settings(char *cmdline_config_path)
                 r->body = ini_get_string(cur_section, "body", r->body);
                 r->icon = ini_get_string(cur_section, "icon", r->icon);
                 r->category = ini_get_string(cur_section, "category", r->category);
-                r->timeout = ini_get_int(cur_section, "timeout", r->timeout);
+                r->timeout = ini_get_time(cur_section, "timeout", r->timeout);
 
                 {
                         char *c = ini_get_string(
@@ -620,7 +671,9 @@ void load_settings(char *cmdline_config_path)
                 r->format = ini_get_string(cur_section, "format", r->format);
                 r->new_icon = ini_get_string(cur_section, "new_icon", r->new_icon);
                 r->history_ignore = ini_get_bool(cur_section, "history_ignore", r->history_ignore);
-                r->script = ini_get_string(cur_section, "script", NULL);
+                r->match_transient = ini_get_bool(cur_section, "match_transient", r->match_transient);
+                r->set_transient = ini_get_bool(cur_section, "set_transient", r->set_transient);
+                r->script = ini_get_path(cur_section, "script", NULL);
         }
 
 #ifndef STATIC_CONFIG
