@@ -7,6 +7,7 @@
 #include "x11/x.h"
 
 enum alignment { left, center, right };
+enum ellipsize { start, middle, end };
 enum icon_position_t { icons_left, icons_right, icons_off };
 enum separator_color { FOREGROUND, AUTO, FRAME, CUSTOM };
 enum follow_mode { FOLLOW_NONE, FOLLOW_MOUSE, FOLLOW_KEYBOARD };
@@ -29,7 +30,7 @@ typedef struct _settings {
         char *lowfgcolor;
         char *lowframecolor;
         char *format;
-        int timeouts[3];
+        gint64 timeouts[3];
         char *icons[3];
         unsigned int transparency;
         char *geom;
@@ -38,13 +39,14 @@ typedef struct _settings {
         int shrink;
         int sort;
         int indicate_hidden;
-        int idle_threshold;
-        int show_age_threshold;
+        gint64 idle_threshold;
+        gint64 show_age_threshold;
         enum alignment align;
         int sticky_history;
         int history_length;
         int show_indicators;
         int word_wrap;
+        enum ellipsize ellipsize;
         int ignore_newline;
         int line_height;
         int notification_height;
@@ -53,7 +55,6 @@ typedef struct _settings {
         int h_padding;
         enum separator_color sep_color;
         char *sep_custom_color_str;
-        char *sep_color_str;
         int frame_width;
         char *frame_color;
         int startup_notification;
@@ -63,7 +64,7 @@ typedef struct _settings {
         char *browser;
         enum icon_position_t icon_position;
         int max_icon_size;
-        char *icon_folders;
+        char *icon_path;
         enum follow_mode f_mode;
         bool always_run_script;
         keyboard_shortcut close_ks;
