@@ -6,27 +6,29 @@
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#include <cairo.h>
+#include <assert.h>
 #include <cairo-xlib.h>
-#include <gdk/gdk.h>
+#include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib-object.h>
 #include <locale.h>
 #include <math.h>
-#include <pango/pangocairo.h>
 #include <pango/pango-attributes.h>
 #include <pango/pango-font.h>
 #include <pango/pango-layout.h>
 #include <pango/pango-types.h>
+#include <pango/pangocairo.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include "src/dbus.h"
 #include "src/dunst.h"
 #include "src/markup.h"
 #include "src/notification.h"
+#include "src/queues.h"
 #include "src/settings.h"
 #include "src/utils.h"
 
@@ -115,24 +117,23 @@ static color_t calculate_foreground_color(color_t bg)
         return color;
 }
 
-
 static color_t x_get_separator_color(colored_layout *cl, colored_layout *cl_next)
 {
         switch (settings.sep_color) {
-                case FRAME:
-                        if (cl_next->n->urgency > cl->n->urgency)
-                                return cl_next->frame;
-                        else
-                                return cl->frame;
-                case CUSTOM:
-                        return x_string_to_color_t(settings.sep_custom_color_str);
-                case FOREGROUND:
-                        return cl->fg;
-                case AUTO:
-                        return calculate_foreground_color(cl->bg);
-                default:
-                        printf("Unknown separator color type. Please file a Bugreport.\n");
-                        return cl->fg;
+        case FRAME:
+                if (cl_next->n->urgency > cl->n->urgency)
+                        return cl_next->frame;
+                else
+                        return cl->frame;
+        case CUSTOM:
+                return x_string_to_color_t(settings.sep_custom_color_str);
+        case FOREGROUND:
+                return cl->fg;
+        case AUTO:
+                return calculate_foreground_color(cl->bg);
+        default:
+                printf("Unknown separator color type. Please file a Bugreport.\n");
+                return cl->fg;
 
         }
 }
@@ -156,16 +157,16 @@ static void r_setup_pango_layout(PangoLayout *layout, int width)
 
         PangoAlignment align;
         switch (settings.align) {
-                case left:
-                default:
-                        align = PANGO_ALIGN_LEFT;
-                        break;
-                case center:
-                        align = PANGO_ALIGN_CENTER;
-                        break;
-                case right:
-                        align = PANGO_ALIGN_RIGHT;
-                        break;
+        case left:
+        default:
+                align = PANGO_ALIGN_LEFT;
+                break;
+        case center:
+                align = PANGO_ALIGN_CENTER;
+                break;
+        case right:
+                align = PANGO_ALIGN_RIGHT;
+                break;
         }
         pango_layout_set_alignment(layout, align);
 
@@ -186,7 +187,8 @@ static bool have_dynamic_width(void)
         return (xctx.geometry.mask & WidthValue && xctx.geometry.w == 0);
 }
 
-static bool does_file_exist(const char *filename){
+static bool does_file_exist(const char *filename)
+{
         return (access(filename, F_OK) != -1);
 }
 
@@ -195,9 +197,10 @@ static bool is_readable_file(const char *filename)
         return (access(filename, R_OK) != -1);
 }
 
-const char *get_filename_ext(const char *filename) {
+const char *get_filename_ext(const char *filename)
+{
         const char *dot = strrchr(filename, '.');
-        if(!dot || dot == filename) return "";
+        if (!dot || dot == filename) return "";
         return dot + 1;
 }
 
@@ -284,21 +287,39 @@ static dimension_t calculate_dimensions(GSList *layouts)
         return dim;
 }
 
-static cairo_surface_t *gdk_pixbuf_to_cairo_surface(const GdkPixbuf *pixbuf)
+static cairo_status_t read_from_buf(void *closure, unsigned char *data, unsigned int size)
 {
+        GByteArray *buf = (GByteArray *)closure;
+
+        unsigned int cpy = MIN(size, buf->len);
+        memcpy(data, buf->data, cpy);
+        g_byte_array_remove_range(buf, 0, cpy);
+
+        return CAIRO_STATUS_SUCCESS;
+}
+
+
+static cairo_surface_t *gdk_pixbuf_to_cairo_surface(GdkPixbuf *pixbuf)
+{
+        /*
+         * Export the gdk pixbuf into buffer as a png and import the png buffer
+         * via cairo again as a cairo_surface_t.
+         * It looks counterintuitive, as there is gdk_cairo_set_source_pixbuf,
+         * which does the job faster. But this would require gtk3 as a dependency
+         * for a single function call. See discussion in #334 and #376.
+         */
         cairo_surface_t *icon_surface = NULL;
-        cairo_t *cr;
-        cairo_format_t format;
-        double width, height;
+        GByteArray *buffer;
+        char *bufstr;
+        gsize buflen;
+
+        gdk_pixbuf_save_to_buffer(pixbuf, &bufstr, &buflen, "png", NULL, NULL);
+
+        buffer = g_byte_array_new_take((guint8*)bufstr, buflen);
+        icon_surface = cairo_image_surface_create_from_png_stream(read_from_buf, buffer);
+
+        g_byte_array_free(buffer, TRUE);
 
-        format = gdk_pixbuf_get_has_alpha(pixbuf) ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24;
-        width = gdk_pixbuf_get_width(pixbuf);
-        height = gdk_pixbuf_get_height(pixbuf);
-        icon_surface = cairo_image_surface_create(format, width, height);
-        cr = cairo_create(icon_surface);
-        gdk_cairo_set_source_pixbuf(cr, pixbuf, 0, 0);
-        cairo_paint(cr);
-        cairo_destroy(cr);
         return icon_surface;
 }
 
@@ -329,18 +350,19 @@ static GdkPixbuf *get_pixbuf_from_path(char *icon_path)
                 if (icon_path[0] == '/' || icon_path[0] == '~') {
                         pixbuf = get_pixbuf_from_file(icon_path);
                 }
-                /* search in icon_folders */
+                /* search in icon_path */
                 if (pixbuf == NULL) {
-                        char *start = settings.icon_folders,
+                        char *start = settings.icon_path,
                              *end, *current_folder, *maybe_icon_path;
                         do {
                                 end = strchr(start, ':');
-                                if (end == NULL) end = strchr(settings.icon_folders, '\0'); /* end = end of string */
+                                if (end == NULL) end = strchr(settings.icon_path, '\0'); /* end = end of string */
 
                                 current_folder = g_strndup(start, end - start);
                                 /* try svg */
                                 maybe_icon_path = g_strconcat(current_folder, "/", icon_path, ".svg", NULL);
                                 if (!does_file_exist(maybe_icon_path)) {
+                                        g_free(maybe_icon_path);
                                         /* fallback to png */
                                         maybe_icon_path = g_strconcat(current_folder, "/", icon_path, ".png", NULL);
                                 }
@@ -403,13 +425,30 @@ static colored_layout *r_init_shared(cairo_t *c, notification *n)
         cl->l = create_layout(c);
 
         if (!settings.word_wrap) {
-                pango_layout_set_ellipsize(cl->l, PANGO_ELLIPSIZE_MIDDLE);
+                PangoEllipsizeMode ellipsize;
+                switch (settings.ellipsize) {
+                case start:
+                        ellipsize = PANGO_ELLIPSIZE_START;
+                        break;
+                case middle:
+                        ellipsize = PANGO_ELLIPSIZE_MIDDLE;
+                        break;
+                case end:
+                        ellipsize = PANGO_ELLIPSIZE_END;
+                        break;
+                default:
+                        assert(false);
+                }
+                pango_layout_set_ellipsize(cl->l, ellipsize);
         }
 
         GdkPixbuf *pixbuf = NULL;
 
-        if (n->raw_icon && settings.icon_position != icons_off) {
+        if (n->raw_icon &&
+            settings.icon_position != icons_off) {
+
                 pixbuf = get_pixbuf_from_raw_image(n->raw_icon);
+
         } else if (n->icon && settings.icon_position != icons_off) {
                 pixbuf = get_pixbuf_from_path(n->icon);
         }
@@ -446,9 +485,9 @@ static colored_layout *r_init_shared(cairo_t *c, notification *n)
                 cl->icon = NULL;
         }
 
-        cl->fg = x_string_to_color_t(n->color_strings[ColFG]);
-        cl->bg = x_string_to_color_t(n->color_strings[ColBG]);
-        cl->frame = x_string_to_color_t(n->color_strings[ColFrame]);
+        cl->fg = x_string_to_color_t(n->colors[ColFG]);
+        cl->bg = x_string_to_color_t(n->colors[ColBG]);
+        cl->frame = x_string_to_color_t(n->colors[ColFrame]);
 
         cl->n = n;
 
@@ -469,11 +508,11 @@ static colored_layout *r_init_shared(cairo_t *c, notification *n)
 
 static colored_layout *r_create_layout_for_xmore(cairo_t *c, notification *n, int qlen)
 {
-       colored_layout *cl = r_init_shared(c, n);
-       cl->text = g_strdup_printf("(%d more)", qlen);
-       cl->attr = NULL;
-       pango_layout_set_text(cl->l, cl->text, -1);
-       return cl;
+        colored_layout *cl = r_init_shared(c, n);
+        cl->text = g_strdup_printf("(%d more)", qlen);
+        cl->attr = NULL;
+        pango_layout_set_text(cl->l, cl->text, -1);
+        return cl;
 }
 
 static colored_layout *r_create_layout_from_notification(cairo_t *c, notification *n)
@@ -513,11 +552,11 @@ static GSList *r_create_layouts(cairo_t *c)
 {
         GSList *layouts = NULL;
 
-        int qlen = g_list_length(g_queue_peek_head_link(queue));
+        int qlen = queues_length_waiting();
         bool xmore_is_needed = qlen > 0 && settings.indicate_hidden;
 
         notification *last = NULL;
-        for (GList *iter = g_queue_peek_head_link(displayed);
+        for (const GList *iter = queues_get_displayed();
                         iter; iter = iter->next)
         {
                 notification *n = iter->data;
@@ -591,9 +630,9 @@ static dimension_t x_render_layout(cairo_t *c, colored_layout *cl, colored_layou
 
         bool use_padding = settings.notification_height <= (2 * settings.padding) + h;
         if (use_padding)
-            dim.y += settings.padding;
+                dim.y += settings.padding;
         else
-            dim.y += (int) (ceil(bg_half_height) - pango_offset);
+                dim.y += (int) (ceil(bg_half_height) - pango_offset);
 
         if (cl->icon && settings.icon_position == icons_left) {
                 cairo_move_to(c, settings.frame_width + cairo_image_surface_get_width(cl->icon) + 2 * settings.h_padding, bg_y + settings.padding + h/2 - h_text/2);
@@ -607,9 +646,9 @@ static dimension_t x_render_layout(cairo_t *c, colored_layout *cl, colored_layou
         pango_cairo_update_layout(c, cl->l);
         pango_cairo_show_layout(c, cl->l);
         if (use_padding)
-            dim.y += h + settings.padding;
+                dim.y += h + settings.padding;
         else
-            dim.y += (int) (floor(bg_half_height) + pango_offset);
+                dim.y += (int)(floor(bg_half_height) + pango_offset);
 
         if (settings.separator_height > 0 && !last) {
                 color_t sep_color = x_get_separator_color(cl, cl_next);
@@ -627,7 +666,7 @@ static dimension_t x_render_layout(cairo_t *c, colored_layout *cl, colored_layou
         }
         cairo_move_to(c, settings.h_padding, dim.y);
 
-        if (cl->icon)  {
+        if (cl->icon) {
                 unsigned int image_width = cairo_image_surface_get_width(cl->icon),
                              image_height = cairo_image_surface_get_height(cl->icon),
                              image_x,
@@ -639,9 +678,9 @@ static dimension_t x_render_layout(cairo_t *c, colored_layout *cl, colored_layou
                         image_x = bg_width - settings.h_padding - image_width + settings.frame_width;
                 }
 
-                cairo_set_source_surface (c, cl->icon, image_x, image_y);
-                cairo_rectangle (c, image_x, image_y, image_width, image_height);
-                cairo_fill (c);
+                cairo_set_source_surface(c, cl->icon, image_x, image_y);
+                cairo_rectangle(c, image_x, image_y, image_width, image_height);
+                cairo_fill(c);
         }
 
         return dim;
@@ -679,10 +718,11 @@ void x_win_draw(void)
         cairo_paint(cairo_ctx.context);
         cairo_show_page(cairo_ctx.context);
 
+        XFlush(xctx.dpy);
+
         cairo_destroy(c);
         cairo_surface_destroy(image_surface);
         r_free_layouts(layouts);
-
 }
 
 static void x_win_move(int width, int height)
@@ -690,6 +730,7 @@ static void x_win_move(int width, int height)
 
         int x, y;
         screen_info *scr = get_active_screen();
+        xctx.cur_screen = scr->scr;
         /* calculate window position */
         if (xctx.geometry.mask & XNegative) {
                 x = (scr->dim.x + (scr->dim.w - width)) + xctx.geometry.x;
@@ -711,7 +752,6 @@ static void x_win_move(int width, int height)
                 XResizeWindow(xctx.dpy, xctx.win, width, height);
         }
 
-
         xctx.window_dim.x = x;
         xctx.window_dim.y = y;
         xctx.window_dim.h = height;
@@ -722,8 +762,14 @@ static void setopacity(Window win, unsigned long opacity)
 {
         Atom _NET_WM_WINDOW_OPACITY =
             XInternAtom(xctx.dpy, "_NET_WM_WINDOW_OPACITY", false);
-        XChangeProperty(xctx.dpy, win, _NET_WM_WINDOW_OPACITY, XA_CARDINAL, 32,
-                        PropModeReplace, (unsigned char *)&opacity, 1L);
+        XChangeProperty(xctx.dpy,
+                        win,
+                        _NET_WM_WINDOW_OPACITY,
+                        XA_CARDINAL,
+                        32,
+                        PropModeReplace,
+                        (unsigned char *)&opacity,
+                        1L);
 }
 
 /*
@@ -733,7 +779,7 @@ static KeySym x_numlock_mod()
 {
         static KeyCode nl = 0;
         KeySym sym = 0;
-        XModifierKeymap * map = XGetModifierMapping(xctx.dpy);
+        XModifierKeymap *map = XGetModifierMapping(xctx.dpy);
 
         if (!nl)
                 nl = XKeysymToKeycode(xctx.dpy, XStringToKeysym("Num_Lock"));
@@ -745,30 +791,30 @@ static KeySym x_numlock_mod()
                                  * could count as 'using implementation details',
                                  * so use this large switch. */
                                 switch (mod) {
-                                        case ShiftMapIndex:
-                                                sym = ShiftMask;
-                                                goto end;
-                                        case LockMapIndex:
-                                                sym = LockMask;
-                                                goto end;
-                                        case ControlMapIndex:
-                                                sym = ControlMask;
-                                                goto end;
-                                        case Mod1MapIndex:
-                                                sym = Mod1Mask;
-                                                goto end;
-                                        case Mod2MapIndex:
-                                                sym = Mod2Mask;
-                                                goto end;
-                                        case Mod3MapIndex:
-                                                sym = Mod3Mask;
-                                                goto end;
-                                        case Mod4MapIndex:
-                                                sym = Mod4Mask;
-                                                goto end;
-                                        case Mod5MapIndex:
-                                                sym = Mod5Mask;
-                                                goto end;
+                                case ShiftMapIndex:
+                                        sym = ShiftMask;
+                                        goto end;
+                                case LockMapIndex:
+                                        sym = LockMask;
+                                        goto end;
+                                case ControlMapIndex:
+                                        sym = ControlMask;
+                                        goto end;
+                                case Mod1MapIndex:
+                                        sym = Mod1Mask;
+                                        goto end;
+                                case Mod2MapIndex:
+                                        sym = Mod2Mask;
+                                        goto end;
+                                case Mod3MapIndex:
+                                        sym = Mod3Mask;
+                                        goto end;
+                                case Mod4MapIndex:
+                                        sym = Mod4Mask;
+                                        goto end;
+                                case Mod5MapIndex:
+                                        sym = Mod5Mask;
+                                        goto end;
                                 }
                         }
                 }
@@ -783,7 +829,7 @@ end:
  * Helper function to use glib's mainloop mechanic
  * with Xlib
  */
-gboolean x_mainloop_fd_prepare(GSource * source, gint * timeout)
+gboolean x_mainloop_fd_prepare(GSource *source, gint *timeout)
 {
         if (timeout)
                 *timeout = -1;
@@ -796,7 +842,7 @@ gboolean x_mainloop_fd_prepare(GSource * source, gint * timeout)
  * Helper function to use glib's mainloop mechanic
  * with Xlib
  */
-gboolean x_mainloop_fd_check(GSource * source)
+gboolean x_mainloop_fd_check(GSource *source)
 {
         return XPending(xctx.dpy) > 0;
 }
@@ -804,8 +850,7 @@ gboolean x_mainloop_fd_check(GSource * source)
 /*
  * Main Dispatcher for XEvents
  */
-gboolean x_mainloop_fd_dispatch(GSource * source, GSourceFunc callback,
-                                gpointer user_data)
+gboolean x_mainloop_fd_dispatch(GSource *source, GSourceFunc callback, gpointer user_data)
 {
         XEvent ev;
         unsigned int state;
@@ -823,6 +868,7 @@ gboolean x_mainloop_fd_dispatch(GSource * source, GSourceFunc callback,
                 case ButtonRelease:
                         if (ev.xbutton.window == xctx.win) {
                                 x_handle_click(ev);
+                                wake_up();
                         }
                         break;
                 case KeyPress:
@@ -833,36 +879,47 @@ gboolean x_mainloop_fd_dispatch(GSource * source, GSourceFunc callback,
                             && XLookupKeysym(&ev.xkey,
                                              0) == settings.close_ks.sym
                             && settings.close_ks.mask == state) {
-                                if (displayed) {
-                                        notification *n = g_queue_peek_head(displayed);
-                                        if (n)
-                                                notification_close(n, 2);
+                                const GList *displayed = queues_get_displayed();
+                                if (displayed && displayed->data) {
+                                        queues_notification_close(displayed->data, REASON_USER);
+                                        wake_up();
                                 }
                         }
                         if (settings.history_ks.str
                             && XLookupKeysym(&ev.xkey,
                                              0) == settings.history_ks.sym
                             && settings.history_ks.mask == state) {
-                                history_pop();
+                                queues_history_pop();
+                                wake_up();
                         }
                         if (settings.close_all_ks.str
                             && XLookupKeysym(&ev.xkey,
                                              0) == settings.close_all_ks.sym
                             && settings.close_all_ks.mask == state) {
-                                move_all_to_history();
+                                queues_history_push_all();
+                                wake_up();
                         }
                         if (settings.context_ks.str
                             && XLookupKeysym(&ev.xkey,
                                              0) == settings.context_ks.sym
                             && settings.context_ks.mask == state) {
                                 context_menu();
+                                wake_up();
                         }
                         break;
                 case FocusIn:
                 case FocusOut:
-                case PropertyNotify:
                         wake_up();
                         break;
+                case PropertyNotify:
+                        /* Ignore PropertyNotify, when we're still on the
+                         * same screen. PropertyNotify is only neccessary
+                         * to detect a focus change to another screen
+                         */
+                        if(   settings.f_mode != FOLLOW_NONE
+                           && get_active_screen()->scr != xctx.cur_screen)
+                                wake_up();
+                        break;
                 default:
                         screen_check_event(ev);
                         break;
@@ -881,7 +938,7 @@ bool x_is_idle(void)
         if (settings.idle_threshold == 0) {
                 return false;
         }
-        return xctx.screensaver_info->idle / 1000 > settings.idle_threshold;
+        return xctx.screensaver_info->idle > settings.idle_threshold / 1000;
 }
 
 /* TODO move to x_mainloop_* */
@@ -891,7 +948,7 @@ bool x_is_idle(void)
 static void x_handle_click(XEvent ev)
 {
         if (ev.xbutton.button == Button3) {
-                move_all_to_history();
+                queues_history_push_all();
 
                 return;
         }
@@ -900,7 +957,7 @@ static void x_handle_click(XEvent ev)
                 int y = settings.separator_height;
                 notification *n = NULL;
                 int first = true;
-                for (GList * iter = g_queue_peek_head_link(displayed); iter;
+                for (const GList *iter = queues_get_displayed(); iter;
                      iter = iter->next) {
                         n = iter->data;
                         if (ev.xbutton.y > y && ev.xbutton.y < y + n->displayed_height)
@@ -913,7 +970,7 @@ static void x_handle_click(XEvent ev)
 
                 if (n) {
                         if (ev.xbutton.button == Button1)
-                                notification_close(n, 2);
+                                queues_notification_close(n, REASON_USER);
                         else
                                 notification_do_action(n);
                 }
@@ -956,26 +1013,26 @@ void x_setup(void)
         x_shortcut_grab(&settings.context_ks);
         x_shortcut_ungrab(&settings.context_ks);
 
-        xctx.color_strings[ColFG][LOW] = settings.lowfgcolor;
-        xctx.color_strings[ColFG][NORM] = settings.normfgcolor;
-        xctx.color_strings[ColFG][CRIT] = settings.critfgcolor;
+        xctx.colors[ColFG][URG_LOW] = settings.lowfgcolor;
+        xctx.colors[ColFG][URG_NORM] = settings.normfgcolor;
+        xctx.colors[ColFG][URG_CRIT] = settings.critfgcolor;
 
-        xctx.color_strings[ColBG][LOW] = settings.lowbgcolor;
-        xctx.color_strings[ColBG][NORM] = settings.normbgcolor;
-        xctx.color_strings[ColBG][CRIT] = settings.critbgcolor;
+        xctx.colors[ColBG][URG_LOW] = settings.lowbgcolor;
+        xctx.colors[ColBG][URG_NORM] = settings.normbgcolor;
+        xctx.colors[ColBG][URG_CRIT] = settings.critbgcolor;
 
         if (settings.lowframecolor)
-                xctx.color_strings[ColFrame][LOW] = settings.lowframecolor;
+                xctx.colors[ColFrame][URG_LOW] = settings.lowframecolor;
         else
-                xctx.color_strings[ColFrame][LOW] = settings.frame_color;
+                xctx.colors[ColFrame][URG_LOW] = settings.frame_color;
         if (settings.normframecolor)
-                xctx.color_strings[ColFrame][NORM] = settings.normframecolor;
+                xctx.colors[ColFrame][URG_NORM] = settings.normframecolor;
         else
-                xctx.color_strings[ColFrame][NORM] = settings.frame_color;
+                xctx.colors[ColFrame][URG_NORM] = settings.frame_color;
         if (settings.critframecolor)
-                xctx.color_strings[ColFrame][CRIT] = settings.critframecolor;
+                xctx.colors[ColFrame][URG_CRIT] = settings.critframecolor;
         else
-                xctx.color_strings[ColFrame][CRIT] = settings.frame_color;
+                xctx.colors[ColFrame][URG_CRIT] = settings.frame_color;
 
         /* parse and set xctx.geometry and monitor position */
         if (settings.geom[0] == '-') {
@@ -989,13 +1046,23 @@ void x_setup(void)
                                             &xctx.geometry.x, &xctx.geometry.y,
                                             &xctx.geometry.w, &xctx.geometry.h);
 
+        /* calculate maximum notification count and push information to queue */
+        if (xctx.geometry.h == 0) {
+                queues_displayed_limit(0);
+        } else if (xctx.geometry.h == 1) {
+                queues_displayed_limit(1);
+        } else if (settings.indicate_hidden) {
+                queues_displayed_limit(xctx.geometry.h - 1);
+        } else {
+                queues_displayed_limit(xctx.geometry.h);
+        }
+
         xctx.screensaver_info = XScreenSaverAllocInfo();
 
         init_screens();
         x_win_setup();
         x_cairo_setup();
         x_shortcut_grab(&settings.history_ks);
-
 }
 
 static void x_set_wm(Window win)
@@ -1009,9 +1076,14 @@ static void x_set_wm(Window win)
                 XInternAtom(xctx.dpy, "_NET_WM_NAME", false);
 
         XStoreName(xctx.dpy, win, title);
-        XChangeProperty(xctx.dpy, win, _net_wm_title,
-                XInternAtom(xctx.dpy, "UTF8_STRING", false), 8,
-                PropModeReplace, (unsigned char *) title, strlen(title));
+        XChangeProperty(xctx.dpy,
+                        win,
+                        _net_wm_title,
+                        XInternAtom(xctx.dpy, "UTF8_STRING", false),
+                        8,
+                        PropModeReplace,
+                        (unsigned char *)title,
+                        strlen(title));
 
         /* set window class */
         char *class = settings.class != NULL ? settings.class : "Dunst";
@@ -1026,8 +1098,14 @@ static void x_set_wm(Window win)
         data[0] = XInternAtom(xctx.dpy, "_NET_WM_WINDOW_TYPE_NOTIFICATION", false);
         data[1] = XInternAtom(xctx.dpy, "_NET_WM_WINDOW_TYPE_UTILITY", false);
 
-        XChangeProperty(xctx.dpy, win, net_wm_window_type, XA_ATOM, 32,
-                PropModeReplace, (unsigned char *) data, 2L);
+        XChangeProperty(xctx.dpy,
+                        win,
+                        net_wm_window_type,
+                        XA_ATOM,
+                        32,
+                        PropModeReplace,
+                        (unsigned char *)data,
+                        2L);
 
         /* set state above */
         Atom net_wm_state =
@@ -1063,13 +1141,18 @@ static void x_win_setup(void)
             ButtonReleaseMask | FocusChangeMask| StructureNotifyMask;
 
         screen_info *scr = get_active_screen();
-        xctx.win =
-            XCreateWindow(xctx.dpy, root, scr->dim.x, scr->dim.y, scr->dim.w,
-                          1, 0, DefaultDepth(xctx.dpy,
-                                                       DefaultScreen(xctx.dpy)),
-                          CopyFromParent, DefaultVisual(xctx.dpy,
-                                                        DefaultScreen(xctx.dpy)),
-                          CWOverrideRedirect | CWBackPixmap | CWEventMask, &wa);
+        xctx.win = XCreateWindow(xctx.dpy,
+                                 root,
+                                 scr->dim.x,
+                                 scr->dim.y,
+                                 scr->dim.w,
+                                 1,
+                                 0,
+                                 DefaultDepth(xctx.dpy, DefaultScreen(xctx.dpy)),
+                                 CopyFromParent,
+                                 DefaultVisual(xctx.dpy, DefaultScreen(xctx.dpy)),
+                                 CWOverrideRedirect | CWBackPixmap | CWEventMask,
+                                 &wa);
 
         x_set_wm(xctx.win);
         settings.transparency =
@@ -1090,7 +1173,7 @@ static void x_win_setup(void)
 void x_win_show(void)
 {
         /* window is already mapped or there's nothing to show */
-        if (xctx.visible || g_queue_is_empty(displayed)) {
+        if (xctx.visible || queues_length_displayed() == 0) {
                 return;
         }
 
@@ -1099,8 +1182,16 @@ void x_win_show(void)
         x_shortcut_grab(&settings.context_ks);
 
         x_shortcut_setup_error_handler();
-        XGrabButton(xctx.dpy, AnyButton, AnyModifier, xctx.win, false,
-                    BUTTONMASK, GrabModeAsync, GrabModeSync, None, None);
+        XGrabButton(xctx.dpy,
+                    AnyButton,
+                    AnyModifier,
+                    xctx.win,
+                    false,
+                    BUTTONMASK,
+                    GrabModeAsync,
+                    GrabModeSync,
+                    None,
+                    None);
         if (x_shortcut_tear_down_error_handler()) {
                 fprintf(stderr, "Unable to grab mouse button(s)\n");
         }
@@ -1145,13 +1236,12 @@ KeySym x_shortcut_string_to_mask(const char *str)
                 fprintf(stderr, "Warning: Unknown Modifier: %s\n", str);
                 return 0;
         }
-
 }
 
 /*
  * Error handler for grabbing mouse and keyboard errors.
  */
-static int GrabXErrorHandler(Display * display, XErrorEvent * e)
+static int GrabXErrorHandler(Display *display, XErrorEvent *e)
 {
         dunst_grab_errored = true;
         char err_buf[BUFSIZ];
@@ -1191,7 +1281,7 @@ static int x_shortcut_tear_down_error_handler(void)
 /*
  * Grab the given keyboard shortcut.
  */
-int x_shortcut_grab(keyboard_shortcut * ks)
+int x_shortcut_grab(keyboard_shortcut *ks)
 {
         if (!ks->is_valid)
                 return 1;
@@ -1201,10 +1291,20 @@ int x_shortcut_grab(keyboard_shortcut * ks)
         x_shortcut_setup_error_handler();
 
         if (ks->is_valid) {
-                XGrabKey(xctx.dpy, ks->code, ks->mask, root,
-                         true, GrabModeAsync, GrabModeAsync);
-                XGrabKey(xctx.dpy, ks->code, ks->mask | x_numlock_mod() , root,
-                         true, GrabModeAsync, GrabModeAsync);
+                XGrabKey(xctx.dpy,
+                         ks->code,
+                         ks->mask,
+                         root,
+                         true,
+                         GrabModeAsync,
+                         GrabModeAsync);
+                XGrabKey(xctx.dpy,
+                         ks->code,
+                         ks->mask | x_numlock_mod(),
+                         root,
+                         true,
+                         GrabModeAsync,
+                         GrabModeAsync);
         }
 
         if (x_shortcut_tear_down_error_handler()) {
@@ -1218,7 +1318,7 @@ int x_shortcut_grab(keyboard_shortcut * ks)
 /*
  * Ungrab the given keyboard shortcut.
  */
-void x_shortcut_ungrab(keyboard_shortcut * ks)
+void x_shortcut_ungrab(keyboard_shortcut *ks)
 {
         Window root;
         root = RootWindow(xctx.dpy, DefaultScreen(xctx.dpy));
@@ -1231,7 +1331,7 @@ void x_shortcut_ungrab(keyboard_shortcut * ks)
 /*
  * Initialize the keyboard shortcut.
  */
-void x_shortcut_init(keyboard_shortcut * ks)
+void x_shortcut_init(keyboard_shortcut *ks)
 {
         if (ks == NULL || ks->str == NULL)
                 return;
